@@ -102,17 +102,17 @@ class AbstractMatcher {
     static const int UNK      = 256;        ///< unknown/undefined character meta-char marker
     static const int BOB      = 257;        ///< begin of buffer meta-char marker
     static const int EOB      = EOF;        ///< end of buffer meta-char marker
+    static const size_t BLOCK = 4096;       ///< minimum remaining unused space in the buffer, to prevent excessive shifting
 #ifndef REFLEX_BUFSZ
     static const size_t BUFSZ = (64*1024);  ///< initial buffer size, at least 4096 bytes
 #else
     static const size_t BUFSZ = REFLEX_BUFSZ;
 #endif
 #ifndef REFLEX_BOLSZ
-    static const size_t BOLSZ = (256*1024); ///< max begin of line size till match to retain in memory by growing the buffer
+    static const size_t BOLSZ = (3*BUFSZ);  ///< max begin of line size till match to retain in memory by growing the buffer
 #else
     static const size_t BOLSZ = REFLEX_BOLSZ;
 #endif
-    static const size_t BLOCK = 4096;       ///< minimum remaining unused space in the buffer, to prevent excessive shifting
     static const size_t REDO  = 0x7FFFFFFF; ///< reflex::Matcher::accept() returns "redo" with reflex::Matcher option "A"
     static const size_t EMPTY = 0xFFFFFFFF; ///< accept() returns "empty" last split at end of input
   };
@@ -135,7 +135,10 @@ class AbstractMatcher {
     size_t      num; ///< number of bytes shifted out so far, when buffer shifted
   };
   /// Event handler functor base class to invoke when the buffer contents are shifted out, e.g. for logging the data searched.
-  struct Handler { virtual void operator()(AbstractMatcher&, const char*, size_t, size_t) = 0; };
+  struct Handler {
+    virtual void operator()(AbstractMatcher&, const char*, size_t, size_t) = 0;
+    virtual ~Handler() { };
+  };
  protected:
   /// AbstractMatcher::Options for matcher engines.
   struct Option {
@@ -153,11 +156,25 @@ class AbstractMatcher {
   };
   /// AbstractMatcher::Iterator class for scanning, searching, and splitting input character sequences.
   template<typename T> /// @tparam <T> AbstractMatcher or const AbstractMatcher
-  class Iterator : public std::iterator<std::input_iterator_tag,T> {
+  class Iterator {
     friend class AbstractMatcher;
     friend class Iterator<typename reflex::TypeOp<T>::ConstType>;
     friend class Iterator<typename reflex::TypeOp<T>::NonConstType>;
    public:
+    /// Non-const AbstractMatcher type
+    typedef typename reflex::TypeOp<T>::NonConstType NonConstT;
+    /// Const AbstractMatcher type
+    typedef typename reflex::TypeOp<T>::ConstType ConstT;
+    /// Iterator iterator_category trait.
+    typedef std::input_iterator_tag iterator_category;
+    /// Iterator value_type trait.
+    typedef T value_type;
+    /// Iterator difference_type trait.
+    typedef std::ptrdiff_t difference_type;
+    /// Iterator pointer trait.
+    typedef T* pointer;
+    /// Iterator reference trait.
+    typedef T& reference;
     /// Construct an AbstractMatcher::Iterator such that Iterator() == AbstractMatcher::Operation(*this, method).end().
     Iterator()
       :
@@ -165,7 +182,7 @@ class AbstractMatcher {
         method_()
     { }
     /// Copy constructor.
-    Iterator(const Iterator<typename reflex::TypeOp<T>::NonConstType>& it)
+    Iterator(const Iterator<NonConstT>& it)
       :
         matcher_(it.matcher_),
         method_(it.method_)
@@ -183,13 +200,13 @@ class AbstractMatcher {
       return matcher_;
     }
     /// AbstractMatcher::Iterator equality.
-    bool operator==(const Iterator<typename reflex::TypeOp<T>::ConstType>& rhs) const
+    bool operator==(const Iterator<ConstT>& rhs) const
       /// @returns true if iterator equals RHS
     {
       return matcher_ == rhs.matcher_;
     }
     /// AbstractMatcher::Iterator inequality.
-    bool operator!=(const Iterator<typename reflex::TypeOp<T>::ConstType>& rhs) const
+    bool operator!=(const Iterator<ConstT>& rhs) const
       /// @returns true if iterator does not equal RHS
     {
       return matcher_ != rhs.matcher_;
@@ -212,8 +229,8 @@ class AbstractMatcher {
     }
     /// Construct an AbstractMatcher::Iterator to scan, search, or split an input character sequence.
     Iterator(
-        AbstractMatcher *matcher, ///< iterate over pattern matches with this matcher
-        Method           method)  ///< match using method Const::SCAN, Const::FIND, or Const::SPLIT
+        NonConstT *matcher, ///< iterate over pattern matches with this matcher
+        Method     method)  ///< match using method Const::SCAN, Const::FIND, or Const::SPLIT
       :
         matcher_(matcher),
         method_(method)
@@ -222,8 +239,8 @@ class AbstractMatcher {
         matcher_ = NULL;
     }
    private:
-    AbstractMatcher *matcher_; ///< the matcher used by this iterator
-    Method           method_;  ///< the method for pattern matching by this iterator's matcher
+    NonConstT *matcher_; ///< the matcher used by this iterator
+    Method     method_;  ///< the method for pattern matching by this iterator's matcher
   };
  public:
   typedef AbstractMatcher::Iterator<AbstractMatcher>       iterator;       ///< std::input_iterator for scanning, searching, and splitting input character sequences
@@ -418,11 +435,15 @@ class AbstractMatcher {
     while (in.good()) // there is more to get while good(), e.g. via wrap()
     {
       (void)grow();
-      end_ += get(buf_ + end_, max_ - end_);
+      size_t len = get(buf_ + end_, max_ - end_);
+      if (len == 0)
+        break;
+      end_ += len;
     }
     if (end_ == max_)
       (void)grow(1); // make sure we have room for a final \0
-    return in.eof();
+    eof_ = in.eof();
+    return eof_;
   }
 #if defined(WITH_SPAN)
   /// Set event handler functor to invoke when the buffer contents are shifted out, e.g. for logging the data searched.
@@ -538,10 +559,14 @@ class AbstractMatcher {
   
   /// Returns nonzero capture index (i.e. true) if the entire input matches this matcher's pattern (and internally caches the true/false result to permit repeat invocations).
   inline size_t matches()
-    /// @returns nonzero capture index (i.e. true) if the entire input matched this matcher's pattern, zero (i.e. false) otherwise
+    /// @returns nonzero capture index if the entire input matched this matcher's pattern, zero (i.e. false) otherwise
   {
     if (!mat_ && at_bob())
-      mat_ = match(Const::MATCH) && at_end();
+    {
+      mat_ = match(Const::MATCH);
+      if (!at_end())
+        mat_ = 0;
+    }
     return mat_;
   }
   /// Returns a positive integer (true) indicating the capture index of the matched text in the pattern or zero (false) for a mismatch.
@@ -616,8 +641,7 @@ class AbstractMatcher {
   /// Set or change the starting line number of the last match.
   inline void lineno(size_t n) ///< new line number
   {
-    if (lpb_ < txt_)
-      (void)lineno(); // update lno_ and bol_ (or cno_) before overriding lno_
+    (void)lineno(); // update lno_ and bol_ (or cno_) before overriding lno_
     lno_ = n;
   }
   /// Updates and returns the starting line number of the match in the input character sequence.
@@ -642,7 +666,7 @@ class AbstractMatcher {
       else
       {
         __m128i vlcn = _mm_set1_epi8('\n');
-        while (s + 15 <= t)
+        while (s + 16 <= t)
         {
           __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
           __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
@@ -659,7 +683,7 @@ class AbstractMatcher {
       else
       {
         __m128i vlcn = _mm_set1_epi8('\n');
-        while (s + 15 <= t)
+        while (s + 16 <= t)
         {
           __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
           __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
@@ -670,7 +694,7 @@ class AbstractMatcher {
       }
 #elif defined(HAVE_SSE2)
       __m128i vlcn = _mm_set1_epi8('\n');
-      while (s + 15 <= t)
+      while (s + 16 <= t)
       {
         __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
         __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
@@ -774,6 +798,17 @@ class AbstractMatcher {
   {
     return lineno() + lines() - 1;
   }
+  /// Set or change the starting column number of the last match.
+  inline void columno(size_t n) ///< new column number
+  {
+    (void)lineno(); // update lno_ and bol_ (or cno_) before overriding lno_
+#if defined(WITH_SPAN)
+    cpb_ = txt_;
+#else
+    lpb_ = txt_;
+#endif
+    cno_ = n;
+  }
   /// Updates and returns the starting column number of the matched text, taking tab spacing into account and counting wide characters as one character each
   inline size_t columno()
     /// @returns column number
