@@ -38,7 +38,7 @@
 #define UGREP_HPP
 
 // ugrep version
-#define UGREP_VERSION "3.7.2"
+#define UGREP_VERSION "3.9.2"
 
 // disable mmap because mmap is almost always slower than the file reading speed improvements since 3.0.0
 #define WITH_NO_MMAP
@@ -46,7 +46,7 @@
 // use a task-parallel thread to decompress the stream into a pipe to search, handles archives and increases decompression speed for larger files
 #define WITH_DECOMPRESSION_THREAD
 
-// drain stdin until eof - this is disabled for speed, almost all utilities handle SIGPIPE these days anyway
+// drain stdin until eof to prevent broken pipe signal
 // #define WITH_STDIN_DRAIN
 
 // enable easy-to-use abbreviated ANSI SGR color codes with WITH_EASY_GREP_COLORS
@@ -245,7 +245,12 @@ inline int fopenw_s(FILE **file, const char *filename, const char *mode)
 #if defined(HAVE_F_RDAHEAD)
   if (strchr(mode, 'a') == NULL && strchr(mode, 'w') == NULL)
   {
-    int fd = open(filename, O_RDONLY); // removed O_NOATIME which may fail
+    // removed O_NOATIME which may fail
+#if defined(O_NOCTTY)
+    int fd = open(filename, O_RDONLY | O_NOCTTY);
+#else
+    int fd = open(filename, O_RDONLY);
+#endif
     if (fd < 0)
       return errno;
     fcntl(fd, F_RDAHEAD, 1);
@@ -263,6 +268,40 @@ inline char *getcwd0()
 
 #endif
 
+// UTF-8 multibyte string length (number of UTF-8-encoded Unicode characters) without validity checking
+inline size_t utf8len(const char *s)
+{
+  size_t k = 0;
+  while (*s != '\0')
+    k += ((*s++ & 0xc0) != 0x80);
+  return k;
+}
+
+// UTF-8 multibyte string length (number of UTF-8-encoded Unicode characters) without validity checking
+inline size_t utf8nlen(const char *s, size_t n)
+{
+  size_t k = 0;
+  while (n-- > 0)
+    k += ((*s++ & 0xc0) != 0x80);
+  return k;
+}
+
+// UTF-8 multibyte string advance k UTF-8-encoded characters or until \0
+inline const char *utf8skip(const char *s, size_t k)
+{
+  while (*s != '\0' && k > 0)
+    k -= ((*++s & 0xc0) != 0x80);
+  return s;
+}
+
+// UTF-8 multibyte string of length n advance k UTF-8-encoded characters
+inline const char *utf8skipn(const char *s, size_t n, size_t k)
+{
+  while (n-- > 0 && k > 0)
+    k -= ((*++s & 0xc0) != 0x80);
+  return s;
+}
+
 // platform -- see configure.ac
 #if !defined(PLATFORM)
 # if defined(OS_WIN)
